/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2020 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * pasoe.p
 *
 * pasoe (aka "pacific") info
 *
 * this data collector uses the "OE Manager" REST API, to install that (paths are generic):
 *
 *   proenv> /usr/wrk/oepas3/bin/tcman deploy /usr/dlc/servers/pasoe/extras/oemanager.war
 *
 * Author:
 *
 *	Tom Bascom, Greenfield Technologies
 *	http://www.greenfieldtech.com
 *	September 5, 2003
 *
 */

/* JsonObject does not exist prior to OE11 so code that works with it needs to be hidden from the compiler
 */

&IF DECIMAL(SUBSTRING(PROVERSION,1,INDEX(PROVERSION,".") + 1)) >= 11.0 &THEN

/* block-level on error undo, throw. */

using Progress.Json.ObjectModel.JsonObject.
using Progress.Json.ObjectModel.JsonArray.
using Progress.Json.ObjectModel.JsonDataType.
using Progress.Json.ObjectModel.ObjectModelParser.
using OpenEdge.Core.JsonDataTypeEnum.

&ENDIF

{lib/protop.i}
{lib/tick.i}

define output parameter dcDescription as character no-undo initial "pasoeInfo".

/* 
 * # pasoe.cfg
 * #
 * # to generate encoded credentials:
 * #
 * #   pro -p ./util/up2b64.p
 * #  or
 * #   echo -n "userid:password" | base64		# beware of the unwanted trailing newline when using "base64"
 * #
 * # name host port path credentials options
 * #
 * #    name is an arbitrary - it should uniquely identify the combination of server and "application"
 * #    path is the REST path, not the os path - it is almost always "/oemanager/applications" (no exceptions are currently known)
 * #
 * 
 * # sample1 localhost 10150 /oemanager/applications "dG9tY2F0OnRvbWNhdA=="
 * # sample2 localhost 10151 /oemanager/applications "dG9tY2F0OnRvbWNhdA==" "-ssl -nohostverify"
 * # sample3 localhost 10152 /oemanager/applications "dG9tY2F0OnRvbWNhdA=="
 * 
 */

define temp-table tt_pasoeCfgList
  field zid           as integer   format ">>9" label "Instance Id"
  field pasoeName     as character
  field pasoeHost     as character
  field pasoePort     as character
  field pasoePath     as character
  field pasoeCred     as character
  field pasoeOptions  as character
  field pasoeStatus   as character
  field zAppListQryTM as integer
  index zid-idx is unique primary zid
.

define variable dtlLogName as character no-undo.

define variable lastPASOEListCheck   as datetime-tz no-undo initial ?.
define variable pt_PASOEListCheckInt as integer     no-undo initial 5. /* 60. */

define variable debugPASOETiming as logical no-undo.

/* "applications" found in pasoe instances
 */

define temp-table tt_pasoeAppInfo
  field xid            as integer    format ">>9" label "Id"
  field zid            as integer    format ">>9" label "Instance Id"
  field instanceName   as character  format "x(18)" label "Instance Name"
  field appName        as character  format "x(20)" label "App Name"
  field zAppVersion    as character  format "x(30)"
  field zAppDescription as character format "x(20)"
/*  field appType        as character  format "x(10)" label "Type"		*/			/* useless, always "oe_broker"	*/
  field appStatus      as character  format "x(8)"  label "Status"
  field memoryUse      as decimal    format  ">>>>>>>9.99" initial ? label "Memory (MB)"
  field memSessMin     as decimal    format    ">>>>>9.99" initial ? label "Min"
  field memSessMax     as decimal    format    ">>>>>9.99" initial ? label "Max"
  field memSessAvg     as decimal    format    ">>>>>9.99" initial ? label "Avg"
  field currCnx        as int64      format      ">>>>>>9" initial ? label "CurrCnx"
  field maxCnx         as int64      format      ">>>>>>9" initial ? label "MaxCnx"
  field currSessions   as int64      format     ">>>>>>>9" initial ? label "CurrSess"
/*  field actvSessions   as int64      format     ">>>>>>>9" initial ? label "ActvSess"	*/
  field requests       as int64      format    ">>>>>>>>9" initial ? label "Requests"
  field reads          as int64      format      ">>>>>>9" initial ? label "Reads"
  field writes         as int64      format      ">>>>>>9" initial ? label "Writes"
  field rdErr          as int64      format      ">>>>>>9" initial ? label "RdErr"
  field wrErr          as int64      format      ">>>>>>9" initial ? label "WrErr"
  field waits          as int64      format      ">>>>>>9" initial ? label "Waits"			/* numReserveABLSessionWaits = request queue	*/
  field timeOuts       as int64      format      ">>>>>>9" initial ? label "TMO"
  /*** ***/
  field zRequests      as int64 extent 5
  field zReads         as int64 extent 5
  field zWrites        as int64 extent 5
  field zRdErr         as int64 extent 5
  field zWrErr         as int64 extent 5
  field zWaits         as int64 extent 5
  field zTimeOuts      as int64 extent 5
  /*** ***/
  field zAppInfoQryTm as integer
  field zAgtInfoQryTm as integer
  /*** ***/
  index xid-idx is unique primary xid ascending
  index zid_appName-idx is unique zid appName
.

define stream debugStrm.


define variable seqId as integer no-undo.

procedure updPASOEApp:

  define input parameter pasoeInstance as integer    no-undo.
  define input parameter instanceName  as character  no-undo.
  define input parameter xName         as character  no-undo.
  define input parameter xVersion      as character  no-undo.
  define input parameter xDescription  as character  no-undo.

  find tt_pasoeAppInfo where tt_pasoeAppInfo.zid = pasoeInstance and  tt_pasoeAppInfo.appname = xName no-error.

  if not available tt_pasoeAppInfo then
    do:
      seqId = seqId + 1.
      create tt_pasoeAppInfo.
      assign
        tt_pasoeAppInfo.xid             = seqId
        tt_pasoeAppInfo.zid             = pasoeInstance
        tt_pasoeAppInfo.instanceName    = instanceName
        tt_pasoeAppInfo.appName         = xName
        tt_pasoeAppInfo.appStatus       = "unknown"
        tt_pasoeAppInfo.zAppVersion     = xVersion
        tt_pasoeAppInfo.zAppDescription = xDescription
      .
    end.

  return.

end.


/*
 * {"operation":"GET SESSION-MGR METRICS","outcome":"SUCCESS","result":{
 *   "requests":0,
 *   "writeErrors":0,
 *   "reads":0,
 *   "concurrentConnectedClients":0,
 *   "writes":0,"maxConcurrentClients":0,
 *   "readErrors":0,
 *   "numReserveABLSessionTimeouts":0,
 *   "numReserveABLSessionWaits":0,				/* queue depth			*/
 *   "type":"OE_BROKER",					/* useless, will *always* be oe_broker	*/
 *   "startTime":"2021-10-13T03:26:29.530-04:00",
 *   "accessTime":"2021-10-15T14:04:40.443-04:00"
 * },"errmsg":"","versionStr":"v11.7.10 ( 2021-04-30 )","versionNo":1}
 * 
 */

/* JsonObject does not exist prior to OE11 so code that works with it needs to be hidden from the compiler
 */

&IF DECIMAL(SUBSTRING(PROVERSION,1,INDEX(PROVERSION,".") + 1)) >= 11.0 &THEN

/* get information about the "applications" in a pasoe instance
 */

procedure getAppInfo:

  define input parameter pasoeInstance as integer    no-undo.
  define input parameter instanceName  as character  no-undo.
  define input parameter payload       as JsonObject no-undo.	/* a json array of applications */

  define variable appArray     as JsonArray no-undo.
  define variable xName        as character no-undo.
  define variable xVersion     as character no-undo.
  define variable xDescription as character no-undo.
  define variable ii           as integer no-undo.
  define variable nn           as integer no-undo.

  assign
    appArray = payload:GetJsonArray( 'Application' )
    nn = appArray:Length
  .

  /* this will find new applications and add them. should it remove deleted applications? (i don't _think_ so but maybe if that's a frequent thing it might change?)
   */

  do ii = 1 to nn:

    assign
      xName        = appArray:getJsonObject( ii ):getCharacter( 'name' )
      xVersion     = appArray:getJsonObject( ii ):getCharacter( 'version' )
      xDescription = appArray:getJsonObject( ii ):getCharacter( 'description' )
    .

    find tt_pasoeAppInfo where tt_pasoeAppInfo.zid = pasoeInstance and  tt_pasoeAppInfo.appname = xname no-error.

    if not available tt_pasoeAppInfo then
      run updPASOEApp ( pasoeInstance, instanceName, xName, xVersion, xDescription ).

  end.

  return.

end.


/* GET a REST response from OE Manager
 *
 * i.e.
 *
 * curl -u tomcat:tomcat -X GET http://localhost:10150/oemanager/applications
 *
 */

procedure getOEM:

  define input  parameter host          as character  no-undo.
  define input  parameter port          as character  no-undo.
  define input  parameter path          as character  no-undo.
  define input  parameter cred          as character  no-undo.
  define input  parameter opts          as character  no-undo.
  define output parameter xStatus       as character  no-undo.
  define output parameter result        as longchar   no-undo.

  define variable ii          as integer    no-undo.
  define variable xRspHdr     as character  no-undo.

  fix-codepage( result ) = 'utf-8'.			/* fix-codepage() can only be run when the argument is empty!   */

  do on error undo, leave:

    publish "logMsg" ( 4, substitute( "getOEM( &1, &2, &3, &4, &5 )", host, port, path, cred, opts )).

    if session:batch = no and dbgMode >= 4 then
      output to value( dtlLogName ) append.
      
    run getURL( host, port, path, cred, opts, '', '', '', output result, output xrspHdr ).

    if session:batch = no and dbgMode >= 4 then
      output close.

  end.

  xStatus = entry( 1, xrspHdr, "~n" ).
  xStatus = trim( replace( xstatus, chr(13), " " )).
  if num-entries( xStatus, " " ) = 2 then xStatus = entry( 2, xStatus, " " ).

  publish "logMsg" ( 4, substitute( "getOEM(), xStatus = &1", xStatus )).

  if xStatus <> "200" then				/* success = "HTTP/1.1 200"	*/
    do:

      if xStatus = "" then xStatus = "Error".

      if dbgMode >= 3 then
        do:

          output stream debugStrm to value( substitute( "&1/pasoe.debug", pt_tmpdir )) append.

          display
            xRspHdr view-as editor size 120 by 20 skip
           with
            frame getOEM
            title "getOEM() errors"
            row 3
            column 10
            width 140
            overlay
          .

          do ii = 1 to error-status:num-messages:
            display
              error-status:get-message( ii ) format "x(120)" skip
             with
              frame errorStat
              down
              row 30
              column 10
              width 140
              overlay
            .
            down with frame errorStat.
          end.

          /*** pause. ***/
          hide frame getOEM.
          hide frame errorStat.

          output stream debugStrm close.

        end.

    end.

  return.

end.


procedure gettc_applist:

  define input parameter pasoeInstance as integer    no-undo.
  define input parameter instanceName  as character  no-undo.
  define input parameter host          as character  no-undo.
  define input parameter port          as character  no-undo.
  define input parameter path          as character  no-undo.
  define input parameter cred          as character  no-undo.
  define input parameter opts          as character  no-undo.

  define variable result      as longchar   no-undo.
  define variable xStatus     as character  no-undo.
  define variable pasoeInfo   as JsonObject no-undo.

  fix-codepage( result ) = 'utf-8'.			/* fix-codepage() can only be run when the argument is empty!   */

  find tt_pasoeCfgList where tt_pasoeCfgList.zid = pasoeInstance no-error.
  if not available tt_pasoeCfgList then
    return.

  tt_pasoeCfgList.zAppListQryTM = mtime.

  publish "logMsg" ( 4, "gettc_applist()" ).

  run getOEM( host, port, path, cred, opts, output xStatus, output result ).

  publish "logMsg" ( 4, substitute( "gettc_applist(), xStatus = &1", xStatus )).

  tt_pasoeCfgList.zAppListQryTM = mtime - tt_pasoeCfgList.zAppListQryTM.
  if tt_pasoeCfgList.zAppListQryTM < 0 then
    tt_pasoeCfgList.zAppListQryTM + 86400000.		/* we wrapped around at midnight so add a day...	*/

  tt_pasoeCfgList.pasoeStatus = xStatus.

  if xStatus <> "200" then				/* success = "HTTP/1.1 200", so this is a failure :(	*/
    do:

      publish "logMsg" ( 4, substitute( 'gettc_applist(), run updPASOEApp( &1, &2, "tc_applist" )', pasoeInstance, instanceName )).

      run updPASOEApp ( pasoeInstance, instanceName, "tc_applist", "", "" ).

      find tt_pasoeAppInfo where tt_pasoeAppInfo.instanceName = instanceName and tt_pasoeAppInfo.appName = "tc_applist" no-error.
      if available tt_pasoeAppInfo then
        assign
          tt_pasoeAppInfo.appStatus = xStatus
/*        tt_pasoeAppInfo.appType   = "tomcat"	*/
        .

      for each tt_pasoeAppInfo where tt_pasoeAppInfo.instanceName = instanceName and tt_pasoeAppInfo.appName <> "tc_applist":
        publish "logMsg" ( 5, substitute( "deleting tt_pasoeAppInfo records for &1 &2", tt_pasoeAppInfo.instanceName, tt_pasoeAppInfo.appName )).
        delete tt_pasoeAppInfo.
      end.

      return.

    end.

  /* the happy path!
   */

  find tt_pasoeAppInfo where tt_pasoeAppInfo.instanceName = instanceName and tt_pasoeAppInfo.appName = "tc_applist" no-error.
  if available tt_pasoeAppInfo then
    delete tt_pasoeAppInfo.

  pasoeInfo = cast(( new ObjectModelParser()):Parse( result ), JsonObject ) no-error.

  if error-status:num-messages = 0 and pasoeInfo:Has('result') and pasoeInfo:GetType('result') = JsonDataType:OBJECT then
    do:
      publish "logMsg" ( 4, substitute( "getAppInfo( &1, &2 )", pasoeInstance, instanceName )).
      run getAppInfo( pasoeInstance, instanceName, pasoeInfo:GetJsonObject('result') ).
    end.
   else
    do:

      publish "logMsg" ( 4, "dc/pasoe.p, gettc_applist() invalid JSON, cannot call getAppInfo()" ).

      message now "dc/pasoe.p, gettc_applist() invalid JSON, cannot call getAppInfo()".
      if session:batch = no and dbgMode > 4 then pause.

    end.

  return.

end.


procedure getAppMetrics:

  define input parameter pasoeInstance as integer    no-undo.
  define input parameter appName       as character  no-undo.
  define input parameter host          as character  no-undo.
  define input parameter port          as character  no-undo.
  define input parameter path          as character  no-undo.
  define input parameter cred          as character  no-undo.
  define input parameter opts          as character  no-undo.

  define variable metricPath  as character  no-undo.

  define variable ii          as integer    no-undo.
  define variable result      as longchar   no-undo.
  define variable xStatus     as character  no-undo.
  define variable xRspHdr     as character  no-undo.
  define variable pasoeInfo   as JsonObject no-undo.
  define variable payload     as JsonObject no-undo.

  fix-codepage( result ) = 'utf-8'.			/* fix-codepage() can only be run when the argument is empty!   */

  find tt_pasoeAppInfo where tt_pasoeAppInfo.zid = pasoeInstance and tt_pasoeAppInfo.appName = appName no-error.
  if not available tt_pasoeAppInfo then
    return.

  metricPath = substitute( "&1/&2/metrics", path, appName ).

  publish "logMsg" ( 4, "getAppMetrics()" ).

  run getOEM( host, port, metricPath, cred, opts, output xStatus, output result ).

  publish "logMsg" ( 4, substitute( "getAppMetrics(), xStatus = &1", xStatus )).

  tt_pasoeAppInfo.appStatus = xStatus.

  if xStatus begins "30" then							/* some sort of redirection?	*/
    do:

      if dbgMode >= 3 then
        do:
          message xrspHdr view-as alert-box info.
        end.

    end.

  if xStatus = "200" then							/* success = "HTTP/1.1 200"	*/
    do:

      pasoeInfo = cast(( new ObjectModelParser()):Parse( result ), JsonObject ) no-error.

      if error-status:num-messages = 0 and pasoeInfo:Has('result') and pasoeInfo:GetType('result') = JsonDataType:OBJECT then
        do:

          payload = pasoeInfo:GetJsonObject('result').

          assign
/*          tt_pasoeAppInfo.appType       = payload:getCharacter( 'type' )						*/
            tt_pasoeAppInfo.currCnx       = max( 0, payload:getInt64( 'concurrentConnectedClients' ))
            tt_pasoeAppInfo.maxCnx        = max( 0, payload:getInt64( 'maxConcurrentClients' ))
          .

          if tt_pasoeAppInfo.requests = ? then
            assign
              {lib/init-xrec.i tt_pasoeAppInfo.zRequests payload:getInt64('requests')}
              {lib/init-xrec.i tt_pasoeAppInfo.zReads    payload:getInt64('reads')}
              {lib/init-xrec.i tt_pasoeAppInfo.zWrites   payload:getInt64('writes')}
              {lib/init-xrec.i tt_pasoeAppInfo.zRdErr    payload:getInt64('readErrors')}
              {lib/init-xrec.i tt_pasoeAppInfo.zWrErr    payload:getInt64('writeErrors')}
              {lib/init-xrec.i tt_pasoeAppInfo.zWaits    payload:getInt64('numReserveABLSessionWaits')}
              {lib/init-xrec.i tt_pasoeAppInfo.zTimeOuts payload:getInt64('numReserveABLSessionTimeouts')}
            no-error.
           else
            assign
              {lib/upd-xrec.i tt_pasoeAppInfo.zRequests payload:getInt64('requests')}
              {lib/upd-xrec.i tt_pasoeAppInfo.zReads    payload:getInt64('reads')}
              {lib/upd-xrec.i tt_pasoeAppInfo.zWrites   payload:getInt64('writes')}
              {lib/upd-xrec.i tt_pasoeAppInfo.zRdErr    payload:getInt64('readErrors')}
              {lib/upd-xrec.i tt_pasoeAppInfo.zWrErr    payload:getInt64('writeErrors')}
              {lib/upd-xrec.i tt_pasoeAppInfo.zWaits    payload:getInt64('numReserveABLSessionWaits')}
              {lib/upd-xrec.i tt_pasoeAppInfo.zTimeOuts payload:getInt64('numReserveABLSessionTimeouts')}
            no-error.

            assign
              tt_pasoeAppInfo.requests      = max( 0, ( tt_pasoeAppInfo.zRequests[x] / z ))
              tt_pasoeAppInfo.reads         = max( 0, ( tt_pasoeAppInfo.zReads[x]    / z ))
              tt_pasoeAppInfo.writes        = max( 0, ( tt_pasoeAppInfo.zWrites[x]   / z ))
              tt_pasoeAppInfo.rdErr         = max( 0, tt_pasoeAppInfo.zRdErr[x] )
              tt_pasoeAppInfo.wrErr         = max( 0, tt_pasoeAppInfo.zWrErr[x] )
              tt_pasoeAppInfo.waits         = max( 0, tt_pasoeAppInfo.zWaits[x] )
              tt_pasoeAppInfo.timeOuts      = max( 0, tt_pasoeAppInfo.zTimeOuts[x] )
            no-error.

          publish "logMsg" ( 4, "getAppMetrics(), completed" ).

        end.

    end.

  return.

end.


procedure getAgentList:

  define input parameter pasoeInstance as integer    no-undo.
  define input parameter appName       as character  no-undo.
  define input parameter host          as character  no-undo.
  define input parameter port          as character  no-undo.
  define input parameter path          as character  no-undo.
  define input parameter cred          as character  no-undo.
  define input parameter opts          as character  no-undo.

  define variable metricPath  as character  no-undo.
  define variable agentPID    as character  no-undo.

  define variable ii          as integer    no-undo.
  define variable jj          as integer    no-undo.
  define variable ll          as integer    no-undo.
  define variable nn          as integer    no-undo.
  define variable result      as longchar   no-undo.
  define variable xStatus     as character  no-undo.
  define variable xRspHdr     as character  no-undo.
  define variable pasoeInfo   as JsonObject no-undo.
  define variable payload     as JsonObject no-undo.
  define variable agentArray  as JsonArray  no-undo.
  define variable agentSess   as JsonArray  no-undo.

  define variable mm          as int64      no-undo.
  define variable agentMemory as int64      no-undo.

  fix-codepage( result ) = 'utf-8'.			/* fix-codepage() can only be run when the argument is empty!   */

  publish "logMsg" ( 4, substitute( "getAgentList() for: &1 &2", pasoeInstance, appName )).

  find tt_pasoeAppInfo where tt_pasoeAppInfo.zid = pasoeInstance and tt_pasoeAppInfo.appName = appName no-error.
  if not available tt_pasoeAppInfo then
    do:
      publish "logMsg" ( 4, substitute( "cannot find pasoeAppInfo for: &1 &2", pasoeInstance, appName )).
      return.
    end.

  assign
    tt_pasoeAppInfo.memSessMin   = 999999999999999
    tt_pasoeAppInfo.memSessMax   = 0
    tt_pasoeAppInfo.currSessions = 0
/*  tt_pasoeAppInfo.actvSessions = 0 */
  .

  metricPath = substitute( "&1/&2/agents", path, appName ).

  publish "logMsg" ( 4, "getAgentList()" ).

  run getOEM( host, port, metricPath, cred, opts, output xStatus, output result ).

  publish "logMsg" ( 4, substitute( "getAgentList(), xStatus = &1", xStatus )).

  if xStatus <> "200" then						/* success = "HTTP/1.1 200"	*/
    do:
      publish "logMsg" ( 4, "getAgentList(), unable to get agent list" ).
    end.
   else
    do:

      pasoeInfo = cast(( new ObjectModelParser()):Parse( result ), JsonObject ) no-error.

      if not( error-status:num-messages = 0 and pasoeInfo:Has('result') and pasoeInfo:GetType('result') = JsonDataType:OBJECT ) then
        do:
          publish "logMsg" ( 4, substitute( "problem extracting agentList: pasoeInfo:Has( 'result' ) = &1, isJson = &2", pasoeInfo:Has('result'), pasoeInfo:GetType('result') = JsonDataType:OBJECT )). 
          do jj = 1 to error-status:num-messages:
            publish "logMsg" ( 4, substitute( "&1", error-status:get-message( ii ))).
          end.
        end.
       else
        do:

          assign
            payload    = pasoeInfo:GetJsonObject('result')
            agentArray = payload:GetJsonArray( 'agents' )
            nn = agentArray:Length
          no-error.

          if nn = 0 then
            do:
              publish "logMsg" ( 3, substitute( "agent has no agents", nn )).
              message pasoeInstance appName "has no agents".
              if session:batch = no and dbgMode > 4 then pause.
              /*** pause. ***/
            end.

          publish "logMsg" ( 4, substitute( "agent has &1 PIDs", nn )).

          do ii = 1 to nn:

            publish "logMsg" ( 5, substitute( "agent &1 of &2", ii, nn )).

            agentPID = agentArray:getJsonObject( ii ):getCharacter( 'pid' ).

            metricPath = substitute( "&1/&2/agents/&3/sessions", path, appName, agentPID ).

            run getOEM( host, port, metricPath, cred, opts, output xStatus, output result ).

            publish "logMsg" ( 5, substitute( "agent &1 of &2, xStatus = &3", ii, nn, xStatus )).

            if xStatus <> "200" then						/* success = "HTTP/1.1 200"	*/
              do:
                publish "logMsg" ( 4, substitute( "unable to obtain information for agent &1 of &2, xStatus = &3", ii, nn, xStatus )).
              end.
             else
              do:

                pasoeInfo = cast(( new ObjectModelParser()):Parse( result ), JsonObject ) no-error.

                if not( error-status:num-messages = 0 and pasoeInfo:Has('result') and pasoeInfo:GetType('result') = JsonDataType:OBJECT ) then
                  do:
                    publish "logMsg" ( 4, substitute( "problem extracting agentSession: pasoeInfo:Has( 'result' ) = &1, isJson = &2", pasoeInfo:Has('result'), pasoeInfo:GetType('result') = JsonDataType:OBJECT )). 
                    do jj = 1 to error-status:num-messages:
                      publish "logMsg" ( 4, substitute( "&1", error-status:get-message( ii ))).
                    end.
                  end.
                 else
                  do:

                    payload   = pasoeInfo:GetJsonObject('result').

                    if payload:Has( 'AgentSession' ) = false then
                      do:
                        publish "logMsg" ( 4, substitute( "no AgentSession is available", nn )).
                        message metricPath "no AgentSession is available".
                        if session:batch = no and dbgMode > 4 then pause.
                        /*** pause. ***/
                      end.
                     else
                      do:

                        assign
                          agentSess = payload:GetJsonArray( 'AgentSession' )
                          ll = agentSess:Length
                        no-error.

                        if error-status:num-messages > 0 then
                          do:
                            publish "logMsg" ( 4, substitute( "error retrieving: &1", metricPath )).
                            message "error retrieving" metricPath. 
                            if session:batch = no and dbgMode > 4 then pause.
                            /*** pause. ***/
                            do jj = 1 to error-status:num-messages:
                              publish "logMsg" ( 4, substitute( "&1", error-status:get-message( ii ))).
                            end.
                          end.
                         else
                          do jj = 1 to ll:

                            publish "logMsg" ( 5, substitute( "getAgentList(), gory details: &1 of &2", jj, ll )).

                            payload = agentSess:getJsonObject( jj ) no-error.

                            mm = ?.
                            mm = payload:getInt64( 'SessionMemory' ) no-error.

                            if mm = ? then next.

                            mm = mm / (1024 * 1024 ).

                            assign
                              agentMemory = agentMemory + mm
                              tt_pasoeAppInfo.currSessions = tt_pasoeAppInfo.currSess + 1
                              tt_pasoeAppInfo.memSessMin = min( mm, tt_pasoeAppInfo.memSessMin )
                              tt_pasoeAppInfo.memSessMax = max( mm, tt_pasoeAppInfo.memSessMax )
                            .

                          end.

                      end.

                  end.

              end.

          end.

          tt_pasoeAppInfo.memoryUse  = agentMemory.

          tt_pasoeAppInfo.memSessAvg = tt_pasoeAppInfo.memoryUse / tt_pasoeAppInfo.currSessions.

          if tt_pasoeAppInfo.memSessMin > tt_pasoeAppInfo.memSessMax then
             tt_pasoeAppInfo.memSessMin = 0.

        end.

    end.

  return.

end.

&ENDIF


define variable cfgFile as character no-undo.
define variable cfgDate as date      no-undo.
define variable cfgTime as integer   no-undo.

procedure chkConfig:

  define variable inData as character no-undo extent 64.

  define variable ii as integer no-undo.

  if cfgFile = "" or cfgFile = ? then run findCfgName( "pasoe", input-output cfgFile ).

  file-info:file-name = cfgFile.

  if file-info:full-pathname = ? then
    do:
      cfgFile = "".
      return.
    end.

  if cfgDate = file-info:file-mod-date and cfgTime = file-info:file-mod-time then return.

  /* if we get this far we have found a config file or the timestamp has changed - so we need to read it
   */

  assign
    cfgDate = file-info:file-mod-date
    cfgTime = file-info:file-mod-time
  .

  empty temp-table tt_pasoeCfgList.

  input stream inStrm from value( file-info:full-pathname ).
  cfgLoop: repeat on endkey undo, leave on error undo, leave:

    inData = "".

    do on error undo, leave cfgLoop
       on endkey undo, leave cfgLoop:

      import stream inStrm inData.

    end.

    if inData[1] = "" or inData[1] begins "#" then next.
    create tt_pasoeCfgList.
    assign
      ii = ii + 1
      tt_pasoeCfgList.zid          = ii		/* instance id	*/
      tt_pasoeCfgList.pasoeName    = inData[1]
      tt_pasoeCfgList.pasoeHost    = inData[2]
      tt_pasoeCfgList.pasoePort    = inData[3]
      tt_pasoeCfgList.pasoePath    = inData[4]
      tt_pasoeCfgList.pasoeCred    = inData[5]
      tt_pasoeCfgList.pasoeOptions = inData[6]
      tt_pasoeCfgList.pasoeStatus  = "Unknown"
    .

  end.
  input stream inStrm close.

/***
 *  if ii = 0 then
 *    do:
 *      create tt_pasoeCfgList.
 *      assign
 *        tt_pasoeCfgList.zid          = -1
 *        tt_pasoeCfgList.pasoeName    = "Invalid config"		/* substitute( "Invalid config file &1", file-info:file-name ) */
 *        tt_pasoeCfgList.pasoeStatus  = "Unknown"
 *      .
 *    end.
 ***/

  return.

end.


define variable dbgFileName as character no-undo.

procedure mon-init:

  define variable monName as character no-undo.

  define variable b as handle no-undo.

  dtlLogName = substitute( "&1/pasoe.&2.details", pt_tmpdir, pt_shortname ).

  run setLogName( dtlLogName ).

  if session:batch = no and dbgMode >= 4 then
    do:
      output to value( dtlLogName ).
      output close.
    end.

  publish "logMsg" ( 4, substitute( "pasoe details for &1", pt_shortName )).

  b = ( temp-table tt_pasoeAppInfo:default-buffer-handle ).

  if pt_altWidth = "80" then
    do:
      altFormat( "80", b, "appName",     "x(24)" ).
      altFormat( "80", b, "appStatus",   "x(10)" ).
      altFormat( "80", b, "memoryUse",   ">>>>>9.99" ).
      altFormat( "80", b, "currCnx",     ">>>>>>9" ).
      altFormat( "80", b, "requests",    ">>>>>>9" ).
      altFormat( "80", b, "waits",       ">>>>>>9" ).
      altFormat( "80", b, "timeouts",    ">>>>>>9" ).
    end.

  run chkConfig.

  if os-getenv( "DEBUGPASOETIMING" ) begins "y" then
    do:

      debugPASOETiming = yes.
      run getMonName ( input-output monName ).

      dbgFileName = substitute( "&1/&2.pasoe.timing.&3.&4", pt_tmpdir, monName, pt_shortname, "dbg" ).
      output stream debugStrm to value( dbgFileName ) append.

      file-info:file-name = dbgFileName.
      if file-info:file-size = 0 then
        do:

          put stream debugStrm unformatted
              "TimeStamp                     "
              "PASOE Name          "
              "Instance Name       "
              "App Name            "
              " AppList"
              " AppInfo"
              " AgtInfo"
            skip
          .

          put stream debugStrm unformatted
              "----------------------------- "
              "------------------- "
              "------------------- "
              "------------------- "
              " -------"
              " -------"
              " -------"
            skip
          .

        end.

      output stream debugStrm close.

    end.

  run updTick.

  return.

end.


procedure mon-update:

  define input parameter argList as character no-undo.

  define variable ii as integer no-undo.

  run chkConfig.
          
  for each tt_pasoeCfgList:
    ii = ii + 1.
  end.  
  
  /** ii = 0. **/
  
  run updTick.

  publish "logMsg" ( 4, "=+= mon-update() =+=" ).

  /* PASOE is not available prior to 11.7
   */

  if decimal( substring( proversion, 1, index( proversion, "." ) + 1 )) < 11.7 then
    do:

      empty temp-table tt_pasoeAppInfo.
      create tt_pasoeAppInfo.
      assign
        tt_pasoeAppInfo.xid       = 1
        tt_pasoeAppInfo.appName   = "PASOE Requires 11.7+"	/* "PASOE monitoring is only available with OE11.7 or greater".			*/
        tt_pasoeAppInfo.appStatus = "Unavailable"
      .

      publish "logMsg" ( 4, "PASOE Requires 11.7+" ).

    end.
   else if ii = 0 then						/* if ii = 0 then no PASOE targets are configured in etc/pasoe.cfg		*/
    do:
      publish "logMsg" ( 4, "No PASOE targets are configured in etc/pasoe.cfg" ).
    end.
   else if ii > 0 then						/* if ii = 0 then no PASOE targets are configured in etc/pasoe.cfg		*/
    do:

      if lastPASOEListCheck = ? or interval( now, lastPASOEListCheck, "seconds" ) > pt_PASOEListCheckInt then
        do:

          publish "logMsg" ( 5, substitute( "checkpasoeCfgList: lastCheck = &1 interval = &2 threshold = &3", lastPASOEListCheck, interval( now, lastPASOEListCheck, "seconds" ), pt_PASOEListCheckInt )).

          lastPASOEListCheck = now.
          for each tt_pasoeCfgList:
            run gettc_applist( tt_pasoeCfgList.zid, tt_pasoeCfgList.pasoeName, tt_pasoeCfgList.pasoeHost, tt_pasoeCfgList.pasoePort, tt_pasoeCfgList.pasoePath, tt_pasoeCfgList.pasoeCred, tt_pasoeCfgList.pasoeOptions ).
          end.
        end.

      publish "logMsg" ( 5, "get app metrics" ).

      for each tt_pasoeAppInfo where tt_pasoeAppInfo.appName <> "tc_applist":

        find tt_pasoeCfgList where tt_pasoeCfgList.zid = tt_pasoeAppInfo.zid no-error.

        publish "logMsg" ( 5, substitute( "for each tt_pasoeAppInfo where... &1 &2", tt_pasoeAppInfo.appName, available( tt_pasoeCfgList ))).

        if available tt_pasoeCfgList then
          do:

            publish "logMsg" ( 5, substitute( "get app metrics for: &1", tt_pasoeAppInfo.appName )).

            tt_pasoeAppInfo.zAppInfoQryTM = mtime.

            run getAppMetrics( tt_pasoeAppInfo.zid, tt_pasoeAppInfo.appName, tt_pasoeCfgList.pasoeHost, tt_pasoeCfgList.pasoePort, tt_pasoeCfgList.pasoePath, tt_pasoeCfgList.pasoeCred, tt_pasoeCfgList.pasoeOptions ).

            tt_pasoeAppInfo.zAppInfoQryTM = mtime - tt_pasoeAppInfo.zAppInfoQryTM.
            if tt_pasoeAppInfo.zAppInfoQryTM < 0 then
              tt_pasoeAppInfo.zAppInfoQryTM + 86400000.		/* we wrapped around at midnight so add a day...	*/

            tt_pasoeAppInfo.zAgtInfoQryTM = mtime.

            publish "logMsg" ( 5, substitute( "get agent list for: &1", tt_pasoeAppInfo.appName )).

            run getAgentList( tt_pasoeAppInfo.zid, tt_pasoeAppInfo.appName, tt_pasoeCfgList.pasoeHost, tt_pasoeCfgList.pasoePort, tt_pasoeCfgList.pasoePath, tt_pasoeCfgList.pasoeCred, tt_pasoeCfgList.pasoeOptions ).

            tt_pasoeAppInfo.zAgtInfoQryTM = mtime - tt_pasoeAppInfo.zAgtInfoQryTM.
            if tt_pasoeAppInfo.zAgtInfoQryTM < 0 then
              tt_pasoeAppInfo.zAgtInfoQryTM + 86400000.		/* we wrapped around at midnight so add a day...	*/

            publish "logMsg" ( 5, substitute( "agent query time: &1", tt_pasoeAppInfo.zAgtInfoQryTM )).

          end.

      end.

    end.

  if debugPASOETiming then
    do:
      output stream debugStrm to value( dbgFileName ) append.
      for each tt_pasoeAppInfo where tt_pasoeAppInfo.appName <> "tc_applist":
        find tt_pasoeCfgList where tt_pasoeCfgList.zid = tt_pasoeAppInfo.zid no-error.
        put stream debugStrm unformatted now " "
            tt_pasoeCfgList.pasoeName        format "x(20)"
            tt_pasoeAppInfo.instanceName     format "x(20)"
            tt_pasoeAppInfo.appName          format "x(20)"
            tt_pasoeCfgList.zAppListQryTM    format ">>>>>>>9"
            tt_pasoeAppInfo.zAppInfoQryTM    format ">>>>>>>9"
            tt_pasoeAppInfo.zAgtInfoQryTM    format ">>>>>>>9"
          skip
        .
      end.
      put stream debugStrm unformatted now " ----" skip.
      output stream debugStrm close.
    end.

  publish "logMsg" ( 5, "=-= mon-update() done =-=" ).
  if session:batch = no and dbgMode >= 4 then
    do:
      output to value( dtlLogName ) append.
      put unformatted "" skip(2).
      output close.
    end.


  add2ds( temp-table tt_pasoeCfgList:default-buffer-handle ).
  add2ds( temp-table tt_pasoeAppInfo:default-buffer-handle ).

  return.

end.

return.
