/* osgetwinsize.p
 *
 * get the current window size and report if it is different from the size passed in (yes = changed, no = not changed):
 *   - on UNIX use ioctl() if we can, otherwise shell out and use "stty size"
 *   - for Windows use getConsoleScreenBufferInfo()
 *   - interpreting the Windows data and mapping that to OpenEdge's geometry is mysterious but you will definitely know that someone is screwing around
 *
 * to load:
 *
 *     run lib/osgetwinsize.p persistent.
 *
 * to declare in a calling procedure:
 *
 *     function OS-GetWinSize returns logical ( input-output numRows as integer, input-output numColumns as integer ) in super.
 *
 */


/* some necessary attributes of the CALL object are not supported in version 10.x and libc.so is missing on AIX
 */

define new global shared variable pt_DynLIBC as logical no-undo initial ?.	/* can we use the CALL handle for dynamic DLLs?				*/

session:add-super-procedure( this-procedure ).

return.


define variable callHandle as handle no-undo.

define variable TIOCGWINSZ as integer no-undo initial 0x5413.
define variable winsize    as memptr  no-undo.

define stream inStrm.

function UNIX-GetWinSize returns logical ( input-output numRows as integer, input-output numColumns as integer ):

  if opsys <> "unix" then return false.

  if pt_DynLIBC = no then
    do:

      if session:batch = no then
        do:
          input stream inStrm through value( "stty size" ).
          import stream inStrm numColumns numRows.				/* be careful, stty reports numColumns first, not numRows		*/
          input stream inStrm close.
        end.

    end.
&IF DECIMAL(SUBSTRING(PROVERSION,1,INDEX(PROVERSION,".") + 1)) > 11.0 &THEN	/* the old oe10 compiler will barf on this stuff			*/ 
   else
    do: 

      define variable libraryName as character no-undo.

      /* setting up the call object only needs to happen on first use
       */

      if not valid-object( callHandle ) then
        do:

          set-size( winsize ) = 64.

          libraryName = os-getenv( "LIBOSGETWINSIZE" ).				/* allow the user to override the default library name			*/
          if libraryName = ? or libraryName = "" then
            libraryName = "libc.so.6".

          create call callHandle.						/* describe the DLL call						*/
          assign
            callHandle:library               = libraryName
            callHandle:call-name             = "ioctl"
            callHandle:call-type             = DLL-CALL-TYPE
            callHandle:return-value-dll-type = "long"
            callHandle:num-parameters        = 3
          .

        end.

      /* now that it has been setup the object _should_ be valid but
       * maybe the library name was invalid or some such problem so
       * so be doubly cautious and check again
       */

      if valid-object( callHandle ) then
        do:

          /* extern int ioctl (int __fd, unsigned long int __request, ...)
           */

          callHandle:set-parameter( 1, "long", "input", 0 ).			/* stdin								*/
          callHandle:set-parameter( 2, "long", "input", TIOCGWINSZ ).		/* request type:	#define TIOCGWINSZ        0x5413		*/
          callHandle:set-parameter( 3, "memptr", "input-output", winsize ).	/* pointer to the winsize struct					*/

          callHandle:invoke().

          /*
           * struct winsize {
           *         unsigned short ws_row;
           *         unsigned short ws_col;
           *         unsigned short ws_xpixel;
           *         unsigned short ws_ypixel;
           * };
           */

          if callHandle:return-value = 0 then					/* success!								*/
            assign
              numRows    = get-short( winsize, 1 )
              numColumns = get-short( winsize, 3 )
            .

        end.

    end.
&ENDIF

  return true.

end.


/* Windows
 */

&IF "{&PROCESS-ARCHITECTURE}" = "64" &THEN
  &global-define LONGINT 	int64
 &ELSE
  &global-define LONGINT 	long
&ENDIF


procedure GetStdHandle external "kernel32.dll":
  define input  parameter fileHandle as {&LONGINT}.
  define return parameter result     as {&LONGINT}.
end.


procedure GetConsoleScreenBufferInfo external "kernel32.dll":
  define input         parameter fileHandle as {&LONGINT}.
  define input-output  parameter cbsi       as memptr.
  define return        parameter result     as {&LONGINT}.
end.


define variable stdInput   as int64  no-undo initial ?.
define variable stdOutput  as int64  no-undo initial ?.
define variable stdError   as int64  no-undo initial ?.

define variable cbsi       as memptr no-undo.

function Windows-GetWinSize returns logical ( input-output numRows as integer, input-output numColumns as integer ):

  define variable result as int64 no-undo.

  if opsys <> "Windows" then return false.

  if stdInput = ? then								/* we only need to do this the first time that we are called		*/
    do:
      run getStdHandle( -10, output stdInput ).					/* only Windows would use -10 for stdin					*/
      run getStdHandle( -11, output stdOutput ).				/* only Windows would use -11 for stdout				*/
      run getStdHandle( -12, output stdError ).					/* only Windows would use -12 for stderr				*/
      set-size( cbsi ) = 32.      						/* 32 _seems_ big enough, seems to really only use 20 or so...		*/
    end.

  run getConsoleScreenBufferInfo( stdInput, input-output cbsi, output result ).

  /***

  message result screen-lines current-window:height current-window:width.	/* none of these will change after session startup			*/

  message

    /* window size */
    get-short( cbsi,  1 )	/* columns */
    get-short( cbsi,  3 )	/* rows  - never changes :( */
  
    /* cursor position */
    get-short( cbsi,  5 )
    get-short( cbsi,  7 )

    /* color attributes - 8 bytes? */
    get-short( cbsi,  9 )
    get-short( cbsi, 11 )
    get-short( cbsi, 13 )	/* offset scrolled down from top?	*/
    get-short( cbsi, 15 )

    /* upper left of visible window */
    get-short( cbsi, 17 )	/* changes when window is resized!	*/
    get-short( cbsi, 19 )

    /* lower right of visible window */
    get-short( cbsi, 21 )
    get-short( cbsi, 23 )

    /* max size ??? seems to always be 0? */
    get-short( cbsi, 25 )
    get-short( cbsi, 26 )

  .

   ***/

  /***
  if result <> 1 or numColumns <> get-short( cbsi, 1 ) /* or numRows <> get-short( cbsi, 3 ) */ or screen-lines <> ( get-short( cbsi, 17 ) - 2 ) then
    do:
      message
        result "screen-lines:" screen-lines "window-height" current-window:height "window-width:" current-window:width  skip
        "c:"  get-short( cbsi,  1 )  "r:" get-short( cbsi,  3 ) "cx:" get-short( cbsi,  5 ) "cy:" get-short( cbsi,  7 ) skip
        "ux:" get-short( cbsi, 17 ) "uy:" get-short( cbsi, 19 ) "lx:" get-short( cbsi, 21 ) "ly:" get-short( cbsi, 23 ) skip 
        "mc:" get-short( cbsi, 25 ) "my:" get-short( cbsi, 26 )
       view-as alert-box
      .
    end.
   ***/

  if result = 1 then
    assign
      numColumns = get-short( cbsi, 15 )
      numRows    = get-short( cbsi, 17 ) - get-short( cbsi, 13 )
    .

  return true.

end.


/* call the os-specific routine to get the window size, then determine
 * if the current size is the same as the size when we were called
 */

function OS-GetWinSize returns logical ( input-output numRows as integer, input-output numColumns as integer ):

  define variable oldNumRows    as integer no-undo.				/* previous window numRows						*/
  define variable oldNumColumns as integer no-undo.				/* previous window numColumns						*/

  assign
    oldnumRows    = numRows
    oldnumColumns = numColumns
  .

  if opsys = "unix" then
    UNIX-GetWinSize( input-output numRows, input-output numColumns ).
   else
    Windows-GetWinSize( input-output numRows, input-output numColumns ).

  return ( numRows <> oldNumRows or numColumns <> oldNumColumns ).		/* has the geometry changed?						*/

end.

